from lxml import etree

from accounting_vasco.builder import build_xml
from accounting_vasco.definitions import construct_vasco_json


class Transfer:
    def __init__(self):
        self.entries = None


class Tax:
    VAT_STANDARD = 'VAT_STANDARD'
    VAT_LOW = 'VAT_LOW'
    VAT_SPECIAL_LOW = 'VAT_SPECIAL_LOW'
    VAT_EXEMPT_HOME = 'VAT_EXEMPT_HOME'
    REVERSE_VAT_EU_GOODS = 'REVERSE_VAT_EU_GOODS'
    REVERSE_VAT_EU_SERVICE = 'REVERSE_VAT_EU_SERVICE'
    EXPORT_OTHER_MARKETS_GOODS = 'EXPORT_OTHER_MARKETS_GOODS'

    def __init__(self, vat_type, base, amount):
        self.vat_type = vat_type
        self.base = base
        self.amount = amount

    def get_vasco_vat_type(self):
        if self.vat_type == Tax.VAT_STANDARD:
            return 'ddv_visja_promet_znotraj_SLO'
        elif self.vat_type == Tax.VAT_LOW:
            return 'ddv_nizja_promet_znotraj_SLO'
        elif self.vat_type == Tax.VAT_SPECIAL_LOW:
            return 'ddv_posebna_nizja_promet_znotraj_SLO'
        elif self.vat_type == Tax.VAT_EXEMPT_HOME:
            return 'oproscen_promet_brez_pravice_do_odbitka'
        elif self.vat_type == Tax.REVERSE_VAT_EU_GOODS:
            return 'oproscen_promet_dobave_v_EU'
        elif self.vat_type == Tax.REVERSE_VAT_EU_SERVICE:
            return 'oproscen_promet_dobave_storitev_v_EU'
        elif self.vat_type == Tax.EXPORT_OTHER_MARKETS_GOODS:
            return 'oproscen_promet_izvoz_blaga'

        # Default value
        return 'ne_gre_v_knjigo'

class TaxData:
    def __init__(self, tax_date):
        self.tax_date = tax_date
        self.taxes = []

    def add_tax(self, vat_type, base, amount):
        self.taxes.append(Tax(vat_type=vat_type, base=base, amount=amount))


class Partner:
    def __init__(self, code, name, address, zip, tax_number, vat_id=None, country_iso=None):
        self.code = code
        self.name = name
        self.address = address
        self.zip = zip
        self.tax_number = tax_number
        self.vat_id = vat_id
        self.country_iso = country_iso


class Entry:
    def __init__(self,
                 symbol,
                 document_number,
                 date_document,
                 date_of_service,
                 date_of_entry,
                 document_description,
                 account_code,
                 cost_code=None,
                 cost_code2=None,
                 cost_code3=None,
                 cost_code4=None,
                 debit=None,
                 credit=None,
                 tax_data=None,
                 partner=None,
                 date_due=None,
                 veza=None,
                 invoice_number=None):
        self.symbol = symbol
        self.document_number = document_number
        self.invoice_number = invoice_number
        self.date_document = date_document
        self.date_of_service = date_of_service
        self.date_of_entry = date_of_entry
        self.document_description = document_description
        self.account_code = account_code
        self.cost_code = cost_code
        self.cost_code2 = cost_code2
        self.cost_code3 = cost_code3
        self.cost_code4 = cost_code4
        self.debit = debit
        self.credit = credit
        self.tax_data = tax_data   # Object TaxData
        self.partner = partner
        self.date_due = date_due
        self.veza = veza


class VASCOExport:
    def __init__(self):
        self.entries = []

    def add_entry(self, entry):
        self.entries.append(entry)

    def render_xml(self):
        xml_content = build_xml(construct_vasco_json(self))

        return ("%s%s" % ('<?xml version="1.0" encoding="UTF-8"?>\n',
                          etree.tostring(xml_content,
                                         pretty_print=True,
                                         xml_declaration=False,
                                         ).decode('utf-8')))


import textwrap

from iso3166 import countries


def construct_vasco_json(vasco_export):
    data = {
        '_name': 'Prenos'
    }

    for i, entry in enumerate(vasco_export.entries):
        data[f'entry_{i}'] = build_entry_json(entry)

    return data


def build_entry_json(entry):
    data = {
        '_name': 'Knjizba',
        '_sorting': ['Simbol', 'Dokument', 'Datum_knjizenja', 'Obracunsko_obdobje', 'Datum_dokumenta', 'Opis_dokumenta',
                     'Konto', 'Partner', 'Rok_placila', 'Sm', 'Sm2', 'Sm3', 'Sm4', 'Debet', 'Kredit', 'Ir', 'Veza', 'Racun'],
        'symbol': {
            '_name': 'Simbol',
            '_value': str(entry.symbol)
        },
        'document_name': {
            '_name': 'Dokument',
            '_value': entry.document_number
        },
        'date_of_entry': {
            '_name': 'Datum_knjizenja',
            '_value': entry.date_of_entry.isoformat()
        },
        'accounting_period': {
            '_name': 'Obracunsko_obdobje',
            'month': {
                '_name': 'Mesec',
                '_value': str(entry.date_of_service.month)
            },
            'year': {
                '_name': 'Leto',
                '_value': str(entry.date_of_service.year)
            }
        },
        'date_document': {
            '_name': 'Datum_dokumenta',
            '_value': entry.date_document.isoformat()
        },
        'document_description': {
            '_name': 'Opis_dokumenta',
            '_value': entry.document_description or 'n/a'
        },
        'account': {
            '_name': 'Konto',
            '_value': str(entry.account_code)
        }
    }

    if entry.date_due:
        data['date_due'] = {
            '_name': 'Rok_placila',
            '_value': entry.date_due.isoformat()
        }

    if entry.veza:
        data['veza'] = {
            '_name': 'Veza',
            '_value': entry.veza
        }

    if entry.invoice_number:
        data['invoice_number'] = {
            '_name': 'Racun',
            '_value': entry.invoice_number
        }

    if entry.credit:
        data['credit'] = {
            '_name': 'Kredit',
            '_value': str(entry.credit)
        }
    else:
        data['debit'] = {
            '_name': 'Debet',
            '_value': str(entry.debit)
        }

    if entry.cost_code:
        data['cost_code'] = {
            '_name': 'Sm',
            'code': {
                '_name': 'Sifra',
                '_value': str(entry.cost_code)
            }
        }

    if entry.cost_code2:
        data['cost_code2'] = {
            '_name': 'Sm2',
            'code': {
                '_name': 'Sifra',
                '_value': str(entry.cost_code2)
            }
        }

    if entry.cost_code3:
        data['cost_code'] = {
            '_name': 'Sm3',
            'code': {
                '_name': 'Sifra',
                '_value': str(entry.cost_code3)
            }
        }

    if entry.cost_code4:
        data['cost_code4'] = {
            '_name': 'Sm4',
            'code': {
                '_name': 'Sifra',
                '_value': str(entry.cost_code4)
            }
        }

    # Append IR object if needed
    if entry.tax_data:
        data['tax_data'] = {
            '_name': 'Ir',
            'vat_date': {
                '_name': 'Datum_za_ddv',
                '_value': entry.tax_data.tax_date.isoformat()
            },
        }

        for i, tax in enumerate(entry.tax_data.taxes):
            data['tax_data'][f'vat_{i}'] = {
                '_name': 'Ddv',
                'tax_type': {
                    '_name': 'Vrsta_ddv',
                    '_value': tax.get_vasco_vat_type()
                },
                'base': {
                    '_name': 'Osnova',
                    '_value': str(tax.base)
                },
                'amount': {
                    '_name': 'Znesek',
                    '_value': str(tax.amount)
                }
            }

    if entry.partner:
        data['partner'] = {
            '_name': 'Partner',
            '_sorting': ['Sifra', 'Naziv1', 'Naziv2', 'Naslov', 'Posta', 'Drzava',
                         'Identifikacijska_stevilka', 'Davcna_stevilka'],
            'code': {
                '_name': 'Sifra',
                '_value': str(entry.partner.code)
            },
            'address': {
                '_name': 'Naslov',
                '_value': entry.partner.address[:40]
            },
            'zip': {
                '_name': 'Posta',
                '_value': str(entry.partner.zip)
            },
            'tax_number': {
                '_name': 'Davcna_stevilka',
                '_value': str(entry.partner.tax_number)
            }
        }

        if entry.partner.vat_id:
            data['partner']['vat_id'] = {
                '_name': 'Identifikacijska_stevilka',
                '_value': entry.partner.vat_id
            }

        if entry.partner.country_iso:
            data['partner']['country'] = {
                '_name': 'Drzava',
                '_value': countries.get(entry.partner.country_iso).numeric
            }

        name_split = textwrap.wrap(entry.partner.name, 40, break_long_words=True)

        for i, bn_part in enumerate(name_split):
            i = i + 1  # Start from 1
            data['partner'][f"name_part_{i}"] = {
                '_name': f"Naziv{i}",
                '_value': bn_part
            }

            if i == 2:
                break  # Stop at max length

    return data



from setuptools import setup, find_packages

setup(
    name='accounting_vasco',
    packages=['accounting_vasco'],
    version='0.1.3',
    description='Generate VASCO KN-21 XML export for accounting software ',
    author='Boris Savic',
    author_email='boris70@gmail.com',
    url='https://github.com/boris-savic/python-accounting-vasco',
    download_url='https://github.com/boris-savic/python-accounting-vasco/tarball/0.1.3',
    keywords=['python vasco', 'accounting', 'vasco'],
    classifiers=[],
    install_requires=[
        'lxml>=4.5.1',
        'iso3166>=1.0.1'
    ]
)
